@@ -1,30 +1,44 @@
 """ Module that handles the like features """
+# import built-in & third-party modules
 import random
 import re
 from re import findall
 
-from .constants import MEDIA_PHOTO, MEDIA_CAROUSEL, MEDIA_ALL_TYPES
-from .time_util import sleep
-from .util import format_number
-from .util import add_user_to_blacklist
-from .util import click_element
-from .util import is_private_profile
-from .util import is_page_available
-from .util import update_activity
-from .util import web_address_navigator
-from .util import get_number_of_posts
-from .util import get_action_delay
-from .util import explicit_wait
-from .util import extract_text_from_element
-from .util import evaluate_mandatory_words
-from .quota_supervisor import quota_supervisor
-from .unfollow_util import get_following_status
+# import exceptions
+from selenium.common.exceptions import (
+    NoSuchElementException,
+    StaleElementReferenceException,
+    WebDriverException,
+)
+
+# import InstaPy modules
+from .comment_util import open_comment_section
+from .constants import (
+    MEDIA_ALL_TYPES,
+    MEDIA_CAROUSEL,
+    MEDIA_IGTV,
+    MEDIA_PHOTO,
+    MEDIA_VIDEO,
+)
 from .event import Event
-
-from selenium.common.exceptions import WebDriverException
-from selenium.common.exceptions import NoSuchElementException
-from selenium.common.exceptions import StaleElementReferenceException
-
+from .follow_util import get_following_status
+from .quota_supervisor import quota_supervisor
+from .time_util import sleep
+from .util import (
+    add_user_to_blacklist,
+    click_element,
+    evaluate_mandatory_words,
+    explicit_wait,
+    extract_text_from_element,
+    format_number,
+    get_action_delay,
+    get_additional_data,
+    get_number_of_posts,
+    is_page_available,
+    is_private_profile,
+    update_activity,
+    web_address_navigator,
+)
 from .xpath import read_xpath
 
 
@@ -59,7 +73,7 @@ def get_links_from_feed(browser, amount, num_of_search, logger):
             logger.info("~~~~~~~~~~~~~~~~~~~~~~~~~~~")
 
     except BaseException as e:
-        logger.error("link_elems error {}".format(str(e)))
+        logger.error("link_elems error \n\t{}".format(str(e).encode("utf-8")))
 
     return links
 
@@ -205,7 +219,7 @@ def get_links_for_location(
                     # by changing this number
                     if put_sleep < 1 and filtered_links <= 21:
                         logger.info(
-                            "Cor! Did you send too many requests? ~ let's " "rest some"
+                            "Cor! Did you send too many requests?  ~let's rest some"
                         )
                         sleep(600)
                         put_sleep += 1
@@ -349,7 +363,7 @@ def get_links_for_tag(browser, tag, amount, skip_top_posts, randomize, media, lo
                 sc_rolled += 1
                 sleep(nap)  # if not slept, and internet speed is low,
                 # instagram will only scroll one time, instead of many times
-                # you sent scoll command...
+                # you sent scroll command...
 
             sleep(3)
             links.extend(get_links(browser, tag, logger, media, main_elem))
@@ -374,7 +388,7 @@ def get_links_for_tag(browser, tag, amount, skip_top_posts, randomize, media, lo
                     # by changing this number
                     if put_sleep < 1 and filtered_links <= 21:
                         logger.info(
-                            "Cor! Did you send too many requests? ~ let's " "rest some"
+                            "Cor! Did you send too many requests?  ~let's rest some"
                         )
                         sleep(600)
                         put_sleep += 1
@@ -442,6 +456,11 @@ def get_links_for_username(
     if taggedImages:
         user_link = user_link + "tagged/"
 
+    # if private user, we can get links only if we following
+    following_status, _ = get_following_status(
+        browser, "profile", username, person, None, logger, logfolder
+    )
+
     # Check URL of the webpage, if it already is user's profile page,
     # then do not navigate to it again
     web_address_navigator(browser, user_link)
@@ -453,11 +472,6 @@ def get_links_for_username(
         )
         return False
 
-    # if private user, we can get links only if we following
-    following_status, _ = get_following_status(
-        browser, "profile", username, person, None, logger, logfolder
-    )
-
     # if following_status is None:
     #    browser.wait_for_valid_connection(browser, username, logger)
 
@@ -465,12 +479,17 @@ def get_links_for_username(
     #    browser.wait_for_valid_authorization(browser, username, logger)
 
     is_private = is_private_profile(browser, logger, following_status == "Following")
+
     if (
         is_private is None
         or (is_private is True and following_status not in ["Following", True])
         or (following_status == "Blocked")
     ):
-        logger.info("This user is private and we are not following")
+        logger.info(
+            "This user is private and we are not following. '{}':'{}'".format(
+                is_private, following_status
+            )
+        )
         return False
 
     # Get links
@@ -481,7 +500,7 @@ def get_links_for_username(
 
     if posts_count is not None and amount > posts_count:
         logger.info(
-            "You have requested to get {} posts from {}'s profile page BUT"
+            "You have requested to get {} posts from {}'s profile page but"
             " there only {} posts available :D".format(amount, person, posts_count)
         )
         amount = posts_count
@@ -518,7 +537,7 @@ def get_links_for_username(
 
 
 def get_media_edge_comment_string(media):
-    """AB test (Issue 3712) alters the string for media edge, this resoves it"""
+    """AB test (Issue 3712) alters the string for media edge, this resolves it"""
     options = ["edge_media_to_comment", "edge_media_preview_comment"]
     for option in options:
         try:
@@ -562,22 +581,7 @@ def check_link(
     web_address_navigator(browser, post_link)
 
     # Check if the Post is Valid/Exists
-    try:
-        post_page = browser.execute_script(
-            "return window.__additionalData[Object.keys(window.__additionalData)[0]].data"
-        )
-
-    except WebDriverException:  # handle the possible `entry_data` error
-        try:
-            browser.execute_script("location.reload()")
-            update_activity(browser, state=None)
-
-            post_page = browser.execute_script(
-                "return window._sharedData.entry_data.PostPage[0]"
-            )
-
-        except WebDriverException:
-            post_page = None
+    post_page = get_additional_data(browser)
 
     if post_page is None:
         logger.warning("Unavailable Page: {}".format(post_link.encode("utf-8")))
@@ -585,6 +589,8 @@ def check_link(
 
     # Gets the description of the post's link and checks for the dont_like tags
     graphql = "graphql" in post_page
+    location_name = None
+
     if graphql:
         media = post_page["graphql"]["shortcode_media"]
         is_video = media["is_video"]
@@ -594,34 +600,21 @@ def check_link(
         location = media["location"]
         location_name = location["name"] if location else None
         media_edge_string = get_media_edge_comment_string(media)
-        # double {{ allows us to call .format here:
-        try:
-            browser.execute_script(
-                "window.insta_data = window.__additionalData[Object.keys(window.__additionalData)[0]].data"
-            )
-        except WebDriverException:
-            browser.execute_script(
-                "window.insta_data = window._sharedData.entry_data.PostPage[0]"
-            )
-        owner_comments = browser.execute_script(
-            """
-            latest_comments = window.insta_data.graphql.shortcode_media.{}.edges;
-            if (latest_comments === undefined) {{
-                latest_comments = Array();
-                owner_comments = latest_comments
-                    .filter(item => item.node.owner.username == arguments[0])
-                    .map(item => item.node.text)
-                    .reduce((item, total) => item + '\\n' + total, '');
-                return owner_comments;}}
-            else {{
-                return null;}}
-        """.format(
-                media_edge_string
-            ),
-            user_name,
+        # Gets all comments on media
+        comments = (
+            media[media_edge_string]["edges"]
+            if media[media_edge_string]["edges"]
+            else None
         )
+        owner_comments = ""
+        # Concat all owner comments
+        if comments is not None:
+            for comment in comments:
+                if comment["node"]["owner"]["username"] == user_name:
+                    owner_comments = owner_comments + "\n" + comment["node"]["text"]
 
     else:
+        logger.info("post_page: {}".format(post_page))
         media = post_page[0]["shortcode_media"]
         is_video = media["is_video"]
         user_name = media["owner"]["username"]
@@ -668,7 +661,7 @@ def check_link(
         image_text = "No description"
 
     logger.info("Image from: {}".format(user_name.encode("utf-8")))
-    logger.info("Link: {}".format(post_link.encode("utf-8")))
+    logger.info("Image link: {}".format(post_link.encode("utf-8")))
     logger.info("Description: {}".format(image_text.encode("utf-8")))
 
     # Check if mandatory character set, before adding the location to the text
@@ -678,8 +671,8 @@ def check_link(
                 True,
                 user_name,
                 is_video,
-                "Mandatory language not " "fulfilled",
-                "Not mandatory " "language",
+                "Mandatory language not fulfilled",
+                "Not mandatory language",
             )
 
     # Append location to image_text so we can search through both in one go
@@ -693,8 +686,8 @@ def check_link(
                 True,
                 user_name,
                 is_video,
-                "Mandatory words not " "fulfilled",
-                "Not mandatory " "likes",
+                "Mandatory words not fulfilled",
+                "Not mandatory likes",
             )
 
     image_text_lower = [x.lower() for x in image_text]
@@ -745,8 +738,25 @@ def like_image(browser, username, blacklist, logger, logfolder, total_liked_img)
     if quota_supervisor("likes") == "jump":
         return False, "jumped"
 
+    media = "Image"  # by default
     like_xpath = read_xpath(like_image.__name__, "like")
     unlike_xpath = read_xpath(like_image.__name__, "unlike")
+    play_xpath = read_xpath(like_image.__name__, "play")
+
+    play_elem = browser.find_elements_by_xpath(play_xpath)
+    if len(play_elem) == 1:
+        # This is because now IG is not only Images, User can share Images and
+        # Videos in one post at the same time, it could be Image -> Video or
+        # Video -> Image so we will try to Like the post like one object.
+        media = "Video"
+        comment = read_xpath(open_comment_section.__name__, "comment_elem")
+        element = browser.find_element_by_xpath(comment)
+
+        # Now, move until 'Comment' section to determine the status of post
+        # Notice that some videos comes from TikTok and could have larger size
+        # c'est la vie...
+        logger.info("--> Found 'Play' button for a video, trying to like it")
+        browser.execute_script("arguments[0].scrollIntoView(true);", element)
 
     # find first for like element
     like_elem = browser.find_elements_by_xpath(like_xpath)
@@ -754,6 +764,8 @@ def like_image(browser, username, blacklist, logger, logfolder, total_liked_img)
     if len(like_elem) == 1:
         # sleep real quick right before clicking the element
         sleep(2)
+        logger.info("--> {}...".format(media))
+
         like_elem = browser.find_elements_by_xpath(like_xpath)
         if len(like_elem) > 0:
             click_element(browser, like_elem[0])
@@ -761,7 +773,7 @@ def like_image(browser, username, blacklist, logger, logfolder, total_liked_img)
         liked_elem = browser.find_elements_by_xpath(unlike_xpath)
 
         if len(liked_elem) == 1:
-            logger.info("--> Image Liked!")
+            logger.info("--> {} liked!".format(media))
             Event().liked(username)
             update_activity(
                 browser, action="likes", state=None, logfolder=logfolder, logger=logger
@@ -777,21 +789,23 @@ def like_image(browser, username, blacklist, logger, logfolder, total_liked_img)
             naply = get_action_delay("like")
             sleep(naply)
 
-            # after every 10 liked image do checking on the block
-            if total_liked_img % 10 == 0 and not verify_liked_image(browser, logger):
+            # after liking an image we do check if liking activity was blocked
+            if not verify_liked_image(browser, logger):
                 return False, "block on likes"
 
             return True, "success"
 
         else:
             # if like not seceded wait for 2 min
-            logger.info("--> Image was not able to get Liked! maybe blocked ?")
+            logger.info(
+                "--> {} was not able to get liked! maybe blocked?".format(media)
+            )
             sleep(120)
 
     else:
         liked_elem = browser.find_elements_by_xpath(unlike_xpath)
         if len(liked_elem) == 1:
-            logger.info("--> Image already liked!")
+            logger.info("--> {} already liked!".format(media))
             return False, "already liked"
 
     logger.info("--> Invalid Like Element!")
@@ -809,9 +823,7 @@ def verify_liked_image(browser, logger):
     if len(like_elem) == 1:
         return True
     else:
-        logger.info(
-            "-------- WARNING! Image was NOT liked! " "You have a BLOCK on likes!"
-        )
+        logger.warning("--> Image was NOT liked! You have a BLOCK on likes!")
         return False
 
 
@@ -822,27 +834,13 @@ def get_tags(browser, url):
     # then do not navigate to it again
     web_address_navigator(browser, url)
 
-    try:
-        browser.execute_script(
-            "window.insta_data = window.__additionalData[Object.keys(window.__additionalData)[0]].data"
-        )
-    except WebDriverException:
-        browser.execute_script(
-            "window.insta_data = window._sharedData.entry_data.PostPage[0]"
-        )
+    additional_data = get_additional_data(browser)
+    image_text = additional_data["graphql"]["shortcode_media"]["edge_media_to_caption"][
+        "edges"
+    ][0]["node"]["text"]
 
-    graphql = browser.execute_script("return ('graphql' in window.insta_data)")
-
-    if graphql:
-        image_text = browser.execute_script(
-            "return window.insta_data.graphql."
-            "shortcode_media.edge_media_to_caption.edges[0].node.text"
-        )
-
-    else:
-        image_text = browser.execute_script(
-            "return window.insta_data.media.caption.text"
-        )
+    if not image_text:
+        image_text = ""
 
     tags = findall(r"#\w*", image_text)
 
@@ -851,10 +849,13 @@ def get_tags(browser, url):
 
 def get_links(browser, page, logger, media, element):
     links = []
+    post_href = None
+
     try:
         # Get image links in scope from hashtag, location and other pages
         link_elems = element.find_elements_by_xpath('//a[starts-with(@href, "/p/")]')
-        sleep(2)
+        sleep(random.randint(2, 5))
+
         if link_elems:
             for link_elem in link_elems:
                 try:
@@ -863,66 +864,71 @@ def get_links(browser, page, logger, media, element):
                         "//a[@href='/p/" + post_href.split("/")[-2] + "/']/child::div"
                     )
 
+                    logger.info("Verifying media type: {}".format(media))
+
                     if len(post_elem) == 1 and MEDIA_PHOTO in media:
-                        # Single photo
+                        logger.info("Found media type: {}".format(MEDIA_PHOTO))
                         links.append(post_href)
 
                     if len(post_elem) == 2:
-                        # Carousel or Video
+                        logger.info(
+                            "Found media type: {} - {} - {}".format(
+                                MEDIA_CAROUSEL, MEDIA_VIDEO, MEDIA_IGTV
+                            )
+                        )
+                        # If you see "Cannot detect post media type. Skip https://www.instagram.com/p/CFvUn0gpaMZ/"
+                        # consider updating the @class,'CzVzU', new format types could be added
+                        # Media types from constants.py must be updated here, otherwise the links
+                        # cannot be categorized.
                         post_category = element.find_element_by_xpath(
                             "//a[@href='/p/"
                             + post_href.split("/")[-2]
-                            + "/']/child::div[@class='u7YqG']/child::span"
+                            + "/']/div[contains(@class,'CzVzU')]/child::*/*[name()='svg']"
                         ).get_attribute("aria-label")
 
+                        logger.info("Post category: {}".format(post_category))
+
                         if post_category in media:
                             links.append(post_href)
+
                 except WebDriverException:
-                    logger.info(
-                        "Cannot detect post media type. Skip {}".format(post_href)
-                    )
+                    # If "post_href" is None skip the logger to avoid confusion,
+                    # the links that are not empty will be catched into the next
+                    # loop. Other case, the "post_href" is not empty and needs
+                    # to be displayed to the STDOUT for further review.
+                    if post_href:
+                        logger.info(
+                            "Cannot detect post media type. Skip {}".format(post_href)
+                        )
         else:
             logger.info("'{}' page does not contain a picture".format(page))
+
     except BaseException as e:
-        logger.error("link_elems error {}".format(str(e)))
+        logger.error("link_elems error \n\t{}".format(str(e).encode("utf-8")))
+
+    # This block is intended to provide more information to the InstaPy user, they would like to
+    # know why the Links cannot be "[Un]Liked", I would like to say that first check if the Media
+    # Type is new, second check if the xpath has been updated and finally verify the acct is not
+    # under a cold-down stage.
+    # If the user can use the link outside InstaPy, they would know IG targeted the acct as
+    # automated.
+    for i, link in enumerate(links):
+        logger.info("Links retrieved:: [{}/{}]".format(i + 1, link))
+
     return links
 
 
 def verify_liking(browser, maximum, minimum, logger):
     """Get the amount of existing existing likes and compare it against maximum
     & minimum values defined by user"""
-    try:
-        likes_count = browser.execute_script(
-            "return window.__additionalData[Object.keys(window.__additionalData)[0]].data"
-            ".graphql.shortcode_media.edge_media_preview_like.count"
-        )
 
-    except WebDriverException:
-        try:
-            browser.execute_script("location.reload()")
-            update_activity(browser, state=None)
+    post_page = get_additional_data(browser)
+    likes_count = post_page["graphql"]["shortcode_media"]["edge_media_preview_like"][
+        "count"
+    ]
 
-            likes_count = browser.execute_script(
-                "return window._sharedData.entry_data."
-                "PostPage[0].graphql.shortcode_media.edge_media_preview_like"
-                ".count"
-            )
-
-        except WebDriverException:
-            try:
-                likes_count = browser.find_element_by_css_selector(
-                    "section._1w76c._nlmjy > div > a > span"
-                ).text
-
-                if likes_count:
-                    likes_count = format_number(likes_count)
-                else:
-                    logger.info("Failed to check likes' count  ~empty string\n")
-                    return True
-
-            except NoSuchElementException:
-                logger.info("Failed to check likes' count\n")
-                return True
+    if not likes_count:
+        likes_count = 0
 
     if maximum is not None and likes_count > maximum:
         logger.info(
@@ -932,7 +938,7 @@ def verify_liking(browser, maximum, minimum, logger):
         return False
     elif minimum is not None and likes_count < minimum:
         logger.info(
-            "Not liked this post! ~less likes exist off minumum limit "
+            "Not liked this post! ~less likes exist off minimum limit "
             "at {}".format(likes_count)
         )
         return False
@@ -941,7 +947,7 @@ def verify_liking(browser, maximum, minimum, logger):
 
 
 def like_comment(browser, original_comment_text, logger):
-    """ Like the given comment """
+    """Like the given comment"""
     comments_block_XPath = read_xpath(
         like_comment.__name__, "comments_block"
     )  # quite an efficient
@@ -987,7 +993,7 @@ def like_comment(browser, original_comment_text, logger):
 
     except (NoSuchElementException, StaleElementReferenceException) as exc:
         logger.error(
-            "Error occured while liking a comment.\n\t{}\n\n".format(
+            "Error occurred while liking a comment.\n\t{}".format(
                 str(exc).encode("utf-8")
             )
         )
