@@ -1,46 +1,52 @@
 """ Common utilities """
+# import built-in & third-party modules
 import time
 import datetime
-from math import ceil
-from math import radians
-from math import degrees as rad2deg
-from math import cos
 import random
 import re
 import regex
 import signal
 import os
 import sys
+import csv
+import sqlite3
+import json
+import emoji
+
+from math import ceil
+from math import radians
+from math import cos
+from math import degrees as rad2deg
 from sys import exit as clean_exit
 from platform import system
 from platform import python_version
 from subprocess import call
-import csv
-import sqlite3
-import json
+from random import randint
 from contextlib import contextmanager
 from tempfile import gettempdir
-import emoji
-from emoji.unicode_codes import UNICODE_EMOJI
 from argparse import ArgumentParser
+from bs4 import BeautifulSoup
 
+from emoji.unicode_codes import UNICODE_EMOJI
 from selenium.webdriver.support.ui import WebDriverWait
 from selenium.webdriver.support import expected_conditions as ec
 from selenium.webdriver.common.by import By
 
+# import InstaPy modules
+from .xpath import read_xpath
+from .event import Event
+from .settings import Settings
 from .time_util import sleep
 from .time_util import sleep_actual
 from .database_engine import get_database
 from .quota_supervisor import quota_supervisor
-from .settings import Settings
 
+# import exceptions
 from selenium.common.exceptions import NoSuchElementException
+from selenium.common.exceptions import StaleElementReferenceException
 from selenium.common.exceptions import WebDriverException
 from selenium.common.exceptions import TimeoutException
 
-from .xpath import read_xpath
-from .event import Event
-
 default_profile_pic_instagram = [
     "https://instagram.flas1-2.fna.fbcdn.net/vp"
     "/a8539c22ed9fec8e1c43b538b1ebfd1d/5C5A1A7A/t51.2885-19"
@@ -63,34 +69,24 @@ next_screenshot = 1
 
 
 def is_private_profile(browser, logger, following=True):
-    is_private = None
-    try:
-        is_private = browser.execute_script(
-            "return window.__additionalData[Object.keys(window.__additionalData)[0]]."
-            "data.graphql.user.is_private"
-        )
+    """
+    Verify id account is Private
 
-    except WebDriverException:
-        try:
-            browser.execute_script("location.reload()")
-            update_activity(browser, state=None)
+    :param browser: The selenium webdriver instance
+    :param logger: the logger instance
+    :param following: Not accessed
+    :return: None if profile cannot be verified
+    """
 
-            is_private = browser.execute_script(
-                "return window._sharedData.entry_data."
-                "ProfilePage[0].graphql.user.is_private"
-            )
+    shared_data = get_shared_data(browser)
+    data = shared_data["entry_data"]["ProfilePage"][0]
 
-        except WebDriverException:
-            return None
+    # Sometimes shared_data["entry_data"]["ProfilePage"][0] is empty, but get_additional_data()
+    # fetches all data needed
+    if not data:
+        data = get_additional_data(browser)
 
-    # double check with xpath that should work only when we not follwoing a
-    # user
-    if is_private and not following:
-        logger.info("Is private account you're not following.")
-        body_elem = browser.find_element_by_tag_name("body")
-        is_private = body_elem.find_element_by_xpath(
-            read_xpath(is_private_profile.__name__, "is_private")
-        )
+    is_private = data["graphql"]["user"]["is_private"]
 
     return is_private
 
@@ -135,6 +131,8 @@ def validate_username(
     max_posts,
     skip_private,
     skip_private_percentage,
+    skip_public,
+    skip_public_percentage,
     skip_no_profile_pic,
     skip_no_profile_pic_percentage,
     skip_business,
@@ -181,7 +179,7 @@ def validate_username(
                     "owner's username"
                 )
                 inap_msg = (
-                    "---> Sorry, this page isn't available!\t~either "
+                    "--> Sorry, this page isn't available!\t~either "
                     "link is broken or page is removed\n"
                 )
                 return False, inap_msg
@@ -191,13 +189,12 @@ def validate_username(
         # `username_or_link`, then it is a `username`
 
     if username == own_username:
-        inap_msg = "---> Username '{}' is yours!\t~skipping user\n".format(own_username)
+        inap_msg = "--> Username '{}' is yours!\t~skipping user\n".format(own_username)
         return False, inap_msg
 
     if username in ignore_users:
-        inap_msg = (
-            "---> '{}' is in the `ignore_users` list\t~skipping "
-            "user\n".format(username)
+        inap_msg = "--> '{}' is in the `ignore_users` list\t~skipping " "user\n".format(
+            username
         )
         return False, inap_msg
 
@@ -212,7 +209,7 @@ def validate_username(
                         logger.info("Username in BlackList: {} ".format(username))
                         return (
                             False,
-                            "---> {} is in blacklist  ~skipping "
+                            "--> {} is in blacklist  ~skipping "
                             "user\n".format(username),
                         )
 
@@ -317,7 +314,7 @@ def validate_username(
                                 username,
                                 "potential user"
                                 if not reverse_relationship
-                                else "massive " "follower",
+                                else "massive follower",
                                 truncate_float(relationship_ratio, 2),
                             )
                         )
@@ -333,38 +330,58 @@ def validate_username(
             number_of_posts = getUserData(
                 "graphql.user.edge_owner_to_timeline_media.count", browser
             )
-        except WebDriverException:
-            logger.error("~cannot get number of posts for username")
-            inap_msg = "---> Sorry, couldn't check for number of posts of " "username\n"
+
+            if max_posts:
+                if number_of_posts > max_posts:
+                    inap_msg = (
+                        "Number of posts ({}) of '{}' exceeds the maximum limit "
+                        "given {}\n".format(number_of_posts, username, max_posts)
+                    )
+                    return False, inap_msg
+            if min_posts:
+                if number_of_posts < min_posts:
+                    inap_msg = (
+                        "Number of posts ({}) of '{}' is less than the minimum "
+                        "limit given {}\n".format(number_of_posts, username, min_posts)
+                    )
+                    return False, inap_msg
+        except (WebDriverException, TypeError):  # Catch if number_of_posts is NoneType
+            logger.error("~cannot get number of posts for username".format(username))
+            inap_msg = "--> Sorry, couldn't check for number of posts of username: '{}'\n".format(
+                username
+            )
             return False, inap_msg
-        if max_posts:
-            if number_of_posts > max_posts:
-                inap_msg = (
-                    "Number of posts ({}) of '{}' exceeds the maximum limit "
-                    "given {}\n".format(number_of_posts, username, max_posts)
-                )
-                return False, inap_msg
-        if min_posts:
-            if number_of_posts < min_posts:
-                inap_msg = (
-                    "Number of posts ({}) of '{}' is less than the minimum "
-                    "limit given {}\n".format(number_of_posts, username, min_posts)
-                )
-                return False, inap_msg
 
     # Skip users
 
     # skip private
     if skip_private:
         try:
-            browser.find_element_by_xpath(
-                "//*[contains(text(), 'This Account is Private')]"
-            )
-            is_private = True
-        except NoSuchElementException:
-            is_private = False
-        if is_private and (random.randint(0, 100) <= skip_private_percentage):
-            return False, "{} is private account, by default skip\n".format(username)
+            is_private = getUserData("graphql.user.is_private", browser)
+
+            if is_private and (random.randint(0, 100) <= skip_private_percentage):
+                return False, "{} is private account, by default skip\n".format(
+                    username
+                )
+        except:
+            logger.error("~could not check if profile is public or private")
+            return False, "--> Sorry, could not check if profile is public or private\n"
+
+    # skip public
+    if skip_public:
+        try:
+            is_public = not getUserData("graphql.user.is_private", browser)
+
+            if is_public and (random.randint(0, 100) <= skip_public_percentage):
+                return (
+                    False,
+                    "{} is public account, skip because of configuration\n".format(
+                        username
+                    ),
+                )
+        except:
+            logger.error("~could not check if profile is public or private")
+            return False, "--> Sorry, could not check if profile is public or private\n"
 
     # skip no profile pic
     if skip_no_profile_pic:
@@ -372,7 +389,7 @@ def validate_username(
             profile_pic = getUserData("graphql.user.profile_pic_url", browser)
         except WebDriverException:
             logger.error("~cannot get the post profile pic url")
-            return False, "---> Sorry, couldn't get if user profile pic url\n"
+            return False, "--> Sorry, couldn't get if user profile pic url\n"
         if (
             profile_pic in default_profile_pic_instagram
             or str(profile_pic).find("11906329_960233084022564_1448528159_a.jpg") > 0
@@ -390,13 +407,13 @@ def validate_username(
             logger.error("~cannot get if user has business account active")
             return (
                 False,
-                "---> Sorry, couldn't get if user has business " "account active\n",
+                "--> Sorry, couldn't get if user has business account active\n",
             )
 
         if skip_non_business and not is_business_account:
             return (
                 False,
-                "---> Skiping non business because skip_non_business set to True",
+                "--> Skipping non business because skip_non_business set to True",
             )
 
         if is_business_account:
@@ -404,7 +421,7 @@ def validate_username(
                 category = getUserData("graphql.user.business_category_name", browser)
             except WebDriverException:
                 logger.error("~cannot get category name for user")
-                return False, "---> Sorry, couldn't get category name for " "user\n"
+                return False, "--> Sorry, couldn't get category name for user\n"
 
             if len(skip_business_categories) == 0:
                 # skip if not in dont_include
@@ -434,12 +451,16 @@ def validate_username(
                     )
 
     if len(skip_bio_keyword) > 0 or len(mandatory_bio_keywords) > 0:
+        # Navigate to the target user profile to read the Bio
+        user_link = "https://www.instagram.com/{}/".format(username)
+        web_address_navigator(browser, user_link)
+
         # if contain stop words then skip
         try:
             profile_bio = getUserData("graphql.user.biography", browser).lower()
         except WebDriverException:
-            logger.error("~cannot get user bio")
-            return False, "---> Sorry, couldn't get get user bio " "account active\n"
+            logger.error("~cannot read '{}' bio".format(username))
+            return False, "--> Sorry, couldn't get get user bio account active\n"
         for bio_keyword in skip_bio_keyword:
             if bio_keyword.lower() in profile_bio:
                 return (
@@ -449,7 +470,7 @@ def validate_username(
                     ),
                 )
         # the mandatory keywords applies to the username as well as the bio text
-        if not evaluate_mandatory_words(
+        if mandatory_bio_keywords and not evaluate_mandatory_words(
             username + " " + profile_bio, mandatory_bio_keywords
         ):
             return False, "Mandatory bio keywords not found"
@@ -461,19 +482,41 @@ def validate_username(
 def getUserData(
     query,
     browser,
-    basequery="return window.__additionalData[Object.keys(window.__additionalData)[0]].data.",
+    basequery="no-longer-needed",
 ):
-    try:
-        data = browser.execute_script(basequery + query)
-        return data
-    except WebDriverException:
-        browser.execute_script("location.reload()")
-        update_activity(browser, state=None)
+    shared_data = get_shared_data(browser)
+    data = shared_data["entry_data"]["ProfilePage"][0]
 
-        data = browser.execute_script(
-            "return window._sharedData." "entry_data.ProfilePage[0]." + query
-        )
-        return data
+    # Sometimes shared_data["entry_data"]["ProfilePage"][0] is empty, but get_additional_data()
+    # fetches all data needed
+    if not data:
+        data = get_additional_data(browser)
+
+    if query.find(".") == -1:
+        data = data[query]
+    else:
+        subobjects = query.split(".")
+        for subobject in subobjects:
+            data = data[subobject]
+
+    return data
+
+
+def getMediaData(
+    query,
+    browser,
+):
+    additional_data = get_additional_data(browser)
+    data = additional_data["graphql"]["shortcode_media"]
+
+    if query.find(".") == -1:
+        data = data[query]
+    else:
+        subobjects = query.split(".")
+        for subobject in subobjects:
+            data = data[subobject]
+
+    return data
 
 
 def update_activity(
@@ -512,7 +555,7 @@ def update_activity(
             with open(path, "w") as json_file:
                 json.dump(data, json_file, indent=4)
         except Exception:
-            logger.warn("Unable to update JSON state file")
+            logger.warning("Unable to update JSON state file")
 
     # in case is just a state update and there is no server call
     if action is None:
@@ -602,7 +645,9 @@ def add_user_to_blacklist(username, campaign, action, logger, logfolder):
                 }
             )
     except Exception as err:
-        logger.error("blacklist dictWrite error {}".format(err))
+        logger.error(
+            "blacklist dictWrite error \n\t{}".format(str(err).encode("utf-8"))
+        )
 
     logger.info(
         "--> {} added to blacklist for {} campaign (action: {})".format(
@@ -634,9 +679,7 @@ def get_active_users(browser, username, posts, boundary, logger):
             if topCount_elements:  # prevent an empty string scenario
                 total_posts = format_number(topCount_elements[0].text)
             else:
-                logger.info(
-                    "Failed to get posts count on your profile!  ~empty " "string"
-                )
+                logger.info("Failed to get posts count on your profile!  ~empty string")
                 total_posts = None
         except NoSuchElementException:
             logger.info("Failed to get posts count on your profile!")
@@ -664,13 +707,15 @@ def get_active_users(browser, username, posts, boundary, logger):
     )
     # posts argument is the number of posts to collect usernames
     logger.info(
-        "Getting active users who liked the latest {} posts:\n  {}".format(
+        "Getting active users who liked the latest {} posts:\n {}".format(
             posts, message
         )
     )
 
     count = 1
     checked_posts = 0
+    user_list = []
+
     while count <= posts:
         # load next post
         try:
@@ -697,7 +742,7 @@ def get_active_users(browser, username, posts, boundary, logger):
                     likers_count += 1
                 else:
                     logger.info(
-                        "Failed to get likers count on your post {}  "
+                        "Failed to get likers count on your post {} "
                         "~empty string".format(count)
                     )
                     likers_count = None
@@ -721,7 +766,7 @@ def get_active_users(browser, username, posts, boundary, logger):
 
             except (IndexError, NoSuchElementException):
                 # Video have no likes button / no posts in page
-                logger.info("video found, try next post until we run out of posts")
+                logger.info("Video found, try next post until we run out of posts")
 
                 # edge case of account having only videos,  or last post is a video.
                 if checked_posts >= total_posts:
@@ -740,7 +785,6 @@ def get_active_users(browser, username, posts, boundary, logger):
             scroll_it = True
             try_again = 0
             start_time = time.time()
-            user_list = []
 
             if likers_count:
                 amount = (
@@ -789,11 +833,12 @@ def get_active_users(browser, username, posts, boundary, logger):
                     sleep_actual(1.2)  # old value 5.6
                     sc_rolled += 1
 
-                user_list = get_users_from_dialog(user_list, dialog)
+                user_list = get_users_from_dialog(user_list, dialog, logger)
 
                 # write & update records at Progress Tracker
                 if amount:
                     progress_tracker(len(user_list), amount, start_time, None)
+                    print("\n")
 
                 if boundary is not None:
                     if len(user_list) >= boundary:
@@ -812,8 +857,8 @@ def get_active_users(browser, username, posts, boundary, logger):
                         if try_again <= 1:  # can increase the amount of tries
                             logger.info(
                                 "Failed to get the desired amount of "
-                                "usernames but trying again.."
-                                "\t|> post:{}  |> attempt: {}\n".format(
+                                "usernames but trying again..."
+                                "\t|> post: {} |> attempt: {}\n".format(
                                     posts, try_again + 1
                                 )
                             )
@@ -823,10 +868,10 @@ def get_active_users(browser, username, posts, boundary, logger):
                             nap_it = 4 if try_again == 0 else 7
                             sleep_actual(nap_it)
 
-            user_list = get_users_from_dialog(user_list, dialog)
+            user_list = get_users_from_dialog(user_list, dialog, logger)
 
             logger.info(
-                "Post {}  |  Likers: found {}, catched {}\n\n".format(
+                "Post {} | Likers: found {}, catched {}\n\n".format(
                     count, likers_count, len(user_list)
                 )
             )
@@ -869,7 +914,7 @@ def get_active_users(browser, username, posts, boundary, logger):
 
 
 def delete_line_from_file(filepath, userToDelete, logger):
-    """ Remove user's record from the followed pool file after unfollowing """
+    """Remove user's record from the followed pool file after unfollowing"""
     if not os.path.isfile(filepath):
         # in case of there is no any followed pool file yet
         return 0
@@ -910,8 +955,9 @@ def delete_line_from_file(filepath, userToDelete, logger):
                 os.remove(file_path_old)
 
             except OSError as e:
-                logger.error("Can't remove file_path_old {}".format(str(e)))
-                sleep(5)
+                logger.error(
+                    "Can't remove file_path_old \n\t{}".format(str(e).encode("utf-8"))
+                )
 
         # rename original file to _old
         os.rename(filepath, file_path_old)
@@ -923,7 +969,9 @@ def delete_line_from_file(filepath, userToDelete, logger):
 
             except OSError as e:
                 logger.error(
-                    "Can't rename file_path_Temp to filepath {}".format(str(e))
+                    "Can't rename file_path_Temp to filepath \n\t{}".format(
+                        str(e).encode("utf-8")
+                    )
                 )
                 sleep(5)
 
@@ -931,7 +979,9 @@ def delete_line_from_file(filepath, userToDelete, logger):
         os.remove(file_path_old)
 
     except BaseException as e:
-        logger.error("delete_line_from_file error {}".format(str(e).encode("utf-8")))
+        logger.error(
+            "delete_line_from_file error \n\t{}".format(str(e).encode("utf-8"))
+        )
 
 
 def scroll_bottom(browser, element, range_int):
@@ -949,6 +999,25 @@ def scroll_bottom(browser, element, range_int):
     return
 
 
+def scroll_down(browser, y: int = 50):
+    """
+    Scroll down the page by 50 pixels
+
+    This is intended mainly for accept_follow_requests(), since user could
+    have several request to be accepted. By default only 10 users are shown,
+    but InstaPy user could requested to accept more than 10.
+
+    :param y: number of pixels to be moved
+    """
+
+    browser.execute_script("window.scrollBy(0, {})".format(y))
+    # update server calls
+    update_activity(browser, state=None)
+    sleep(randint(1, 5))
+
+    return
+
+
 def click_element(browser, element, tryNum=0):
     """
     There are three (maybe more) different ways to "click" an element/button.
@@ -963,7 +1032,7 @@ def click_element(browser, element, tryNum=0):
     specific case. See the following issue for more details
     https://github.com/timgrossmann/InstaPy/issues/1232
 
-    explaination of the following recursive function:
+    explanation of the following recursive function:
       we will attempt to click the element given, if an error is thrown
       we know something is wrong (element not in view, element doesn't
       exist, ...). on each attempt try and move the screen around in
@@ -1089,7 +1158,7 @@ def get_number_of_posts(browser):
 
 
 def get_relationship_counts(browser, username, logger):
-    """ Gets the followers & following counts of a given user """
+    """Gets the followers & following counts of a given user"""
 
     user_link = "https://www.instagram.com/{}/".format(username)
 
@@ -1254,6 +1323,8 @@ def interruption_handler(
     if notify is not None and logger is not None:
         logger.warning(notify)
 
+    original_handler = None
+
     if not threaded:
         original_handler = signal.signal(SIG_type, handler)
 
@@ -1268,12 +1339,14 @@ def interruption_handler(
 def highlight_print(
     username=None, message=None, priority=None, level=None, logger=None
 ):
-    """ Print headers in a highlighted style """
+    """Print headers in a highlighted style"""
     # can add other highlighters at other priorities enriching this function
 
     # find the number of chars needed off the length of the logger message
     output_len = 28 + len(username) + 3 + len(message) if logger else len(message)
     show_logs = Settings.show_logs
+    upper_char = None
+    lower_char = None
 
     if priority in ["initialization", "end"]:
         # OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
@@ -1340,7 +1413,7 @@ def highlight_print(
 
 
 def remove_duplicates(container, keep_order, logger):
-    """ Remove duplicates from all kinds of data types easily """
+    """Remove duplicates from all kinds of data types easily"""
     # add support for data types as needed in future
     # currently only 'list' data type is supported
     if isinstance(container, list):
@@ -1364,7 +1437,9 @@ def remove_duplicates(container, keep_order, logger):
 
 
 def dump_record_activity(profile_name, logger, logfolder):
-    """ Dump the record activity data to a local human-readable JSON """
+    """Dump the record activity data to a local human-readable JSON"""
+
+    conn = None
 
     try:
         # get a DB and start a connection
@@ -1486,7 +1561,7 @@ def emergency_exit(browser, username, logger):
 
 
 def load_user_id(username, person, logger, logfolder):
-    """ Load the user ID at reqeust from local records """
+    """Load the user ID at request from local records"""
     pool_name = "{0}{1}_followedPool.csv".format(logfolder, username)
     user_id = "undefined"
 
@@ -1509,7 +1584,7 @@ def load_user_id(username, person, logger, logfolder):
 
     except BaseException as exc:
         logger.exception(
-            "Failed to load the user ID of '{}'!\n{}".format(
+            "Failed to load the user ID of '{}'!\n\t{}".format(
                 person, str(exc).encode("utf-8")
             )
         )
@@ -1518,7 +1593,7 @@ def load_user_id(username, person, logger, logfolder):
 
 
 def check_authorization(browser, username, method, logger, notify=True):
-    """ Check if user is NOW logged in """
+    """Check if user is NOW logged in"""
     if notify is True:
         logger.info("Checking if '{}' is logged in...".format(username))
 
@@ -1579,7 +1654,10 @@ def check_authorization(browser, username, method, logger, notify=True):
 
 
 def get_username(browser, track, logger):
-    """ Get the username of a user from the loaded profile page """
+    """Get the username of a user from the loaded profile page"""
+
+    query = None
+
     if track == "profile":
         query = "return window._sharedData.entry_data. \
                     ProfilePage[0].graphql.user.username"
@@ -1613,7 +1691,11 @@ def get_username(browser, track, logger):
 
 
 def find_user_id(browser, track, username, logger):
-    """  Find the user ID from the loaded page """
+    """Find the user ID from the loaded page"""
+
+    query = None
+    meta_XP = None
+
     logger.info(
         "Attempting to find user ID: Track: {}, Username {}".format(track, username)
     )
@@ -1637,9 +1719,7 @@ def find_user_id(browser, track, username, logger):
             update_activity(browser, state=None)
 
             user_id = browser.execute_script(
-                "return window._sharedData."
-                "entry_data.ProfilePage[0]."
-                "graphql.user.id"
+                "return window._sharedData.entry_data.ProfilePage[0].graphql.user.id"
             )
 
         except WebDriverException:
@@ -1705,6 +1785,9 @@ def explicit_wait(browser, track, ec_params, logger, timeout=35, notify=True):
         ec_params = [ec_params]
 
     # find condition according to the tracks
+    condition = None
+    ec_name = None
+
     if track == "VOEL":
         elem_address, find_method = ec_params
         ec_name = "visibility of element located"
@@ -1755,7 +1838,7 @@ def explicit_wait(browser, track, ec_params, logger, timeout=35, notify=True):
 
 
 def get_current_url(browser):
-    """ Get URL of the loaded webpage """
+    """Get URL of the loaded webpage"""
     try:
         current_url = browser.execute_script("return window.location.href")
 
@@ -1770,14 +1853,14 @@ def get_current_url(browser):
 
 
 def get_username_from_id(browser, user_id, logger):
-    """ Convert user ID to username """
+    """Convert user ID to username"""
     # method using graphql 'Account media' endpoint
     logger.info("Trying to find the username from the given user ID by loading a post")
 
     query_hash = "42323d64886122307be10013ad2dcc44"  # earlier-
     # "472f257a40c653c64c666ce877d59d2b"
-    graphql_query_URL = (
-        "https://www.instagram.com/graphql/query/?query_hash" "={}".format(query_hash)
+    graphql_query_URL = "https://www.instagram.com/graphql/query/?query_hash={}".format(
+        query_hash
     )
     variables = {"id": str(user_id), "first": 1}
     post_url = "{}&variables={}".format(graphql_query_URL, str(json.dumps(variables)))
@@ -1848,7 +1931,7 @@ def get_username_from_id(browser, user_id, logger):
 
 
 def is_page_available(browser, logger):
-    """ Check if the page is available and valid """
+    """Check if the page is available and valid"""
     expected_keywords = ["Page Not Found", "Content Unavailable"]
     page_title = get_page_title(browser, logger)
 
@@ -1865,7 +1948,7 @@ def is_page_available(browser, logger):
 
             elif "Content Unavailable" in page_title:
                 logger.warning(
-                    "The page isn't available!\t~the user may have blocked " "you..."
+                    "The page isn't available!\t~the user may have blocked you..."
                 )
 
             return False
@@ -1899,7 +1982,7 @@ def smart_run(session, threaded=False):
 
 
 def reload_webpage(browser):
-    """ Reload the current webpage """
+    """Reload the current webpage"""
     browser.execute_script("location.reload()")
     update_activity(browser, state=None)
     sleep(2)
@@ -1908,7 +1991,7 @@ def reload_webpage(browser):
 
 
 def get_page_title(browser, logger):
-    """ Get the title of the webpage """
+    """Get the title of the webpage"""
     # wait for the current page fully load to get the correct page's title
     explicit_wait(browser, "PFL", [], logger, 10)
 
@@ -1933,7 +2016,7 @@ def get_page_title(browser, logger):
 
 
 def click_visibly(browser, element):
-    """ Click as the element become visible """
+    """Click as the element become visible"""
     if element.is_displayed():
         click_element(browser, element)
 
@@ -1947,6 +2030,7 @@ def click_visibly(browser, element):
         )
         # update server calls
         update_activity(browser, state=None)
+        sleep(randint(1, 5))
 
         click_element(browser, element)
 
@@ -1954,7 +2038,7 @@ def click_visibly(browser, element):
 
 
 def get_action_delay(action):
-    """ Get the delay time to sleep after doing actions """
+    """Get the delay time to sleep after doing actions"""
     defaults = {"like": 2, "comment": 2, "follow": 3, "unfollow": 10, "story": 3}
     config = Settings.action_delays
 
@@ -2005,7 +2089,7 @@ def get_action_delay(action):
 
 
 def deform_emojis(text):
-    """ Convert unicode emojis into their text form """
+    """Convert unicode emojis into their text form"""
     new_text = ""
     emojiless_text = ""
     data = regex.findall(r"\X", text)
@@ -2035,7 +2119,7 @@ def deform_emojis(text):
 
 
 def extract_text_from_element(elem):
-    """ As an element is valid and contains text, extract it and return """
+    """As an element is valid and contains text, extract it and return"""
     if elem and hasattr(elem, "text") and elem.text:
         text = elem.text
     else:
@@ -2045,7 +2129,7 @@ def extract_text_from_element(elem):
 
 
 def truncate_float(number, precision, round=False):
-    """ Truncate (shorten) a floating point value at given precision """
+    """Truncate (shorten) a floating point value at given precision"""
 
     # don't allow a negative precision [by mistake?]
     precision = abs(precision)
@@ -2070,7 +2154,7 @@ def truncate_float(number, precision, round=False):
 
 
 def get_time_until_next_month():
-    """ Get total seconds remaining until the next month """
+    """Get total seconds remaining until the next month"""
     now = datetime.datetime.now()
     next_month = now.month + 1 if now.month < 12 else 1
     year = now.year if now.month < 12 else now.year + 1
@@ -2082,14 +2166,14 @@ def get_time_until_next_month():
 
 
 def remove_extra_spaces(text):
-    """ Find and remove redundant spaces more than 1 in text """
+    """Find and remove redundant spaces more than 1 in text"""
     new_text = re.sub(r" {2,}", " ", text)
 
     return new_text
 
 
 def has_any_letters(text):
-    """ Check if the text has any letters in it """
+    """Check if the text has any letters in it"""
     # result = re.search("[A-Za-z]", text)   # works only with english letters
     result = any(
         c.isalpha() for c in text
@@ -2157,26 +2241,37 @@ def is_follow_me(browser, person=None):
     return getUserData("graphql.user.follows_viewer", browser)
 
 
-def get_users_from_dialog(old_data, dialog):
+def get_users_from_dialog(old_data, dialog, logger):
     """
     Prepared to work specially with the dynamic data load in the 'Likes'
     dialog box
     """
 
-    user_blocks = dialog.find_elements_by_tag_name("a")
-    loaded_users = [
-        extract_text_from_element(u)
-        for u in user_blocks
-        if extract_text_from_element(u)
-    ]
-    new_data = old_data + loaded_users
-    new_data = remove_duplicates(new_data, True, None)
+    try:
+        user_blocks = dialog.find_elements_by_tag_name("a")
+        loaded_users = [
+            extract_text_from_element(u)
+            for u in user_blocks
+            if extract_text_from_element(u)
+        ]
+        new_data = old_data + loaded_users
+        new_data = remove_duplicates(new_data, True, None)
 
-    return new_data
+        return new_data
+
+    except (NoSuchElementException, StaleElementReferenceException) as exc:
+        # Catch stale elements if any
+        logger.error(
+            "Error occurred while retrieving data.\n\t{}".format(
+                str(exc).encode("utf-8")
+            )
+        )
+
+        return old_data
 
 
 def progress_tracker(current_value, highest_value, initial_time, logger):
-    """ Provide a progress tracker to keep value updated until finishes """
+    """Provide a progress tracker to keep value updated until finishes"""
     if current_value is None or highest_value is None or highest_value == 0:
         return
 
@@ -2215,9 +2310,9 @@ def progress_tracker(current_value, highest_value, initial_time, logger):
         )
 
         total_message = (
-            "\r  {}/{} {}  {}%    "
-            "|> Elapsed: {}    "
-            "|> ETA: {}      ".format(
+            "\r  {0}/{1} {2}  {3}% "
+            "|> Elapsed: {4:20} "
+            "|> ETA: {5:20} ".format(
                 current_value,
                 highest_value,
                 progress_container,
@@ -2236,12 +2331,14 @@ def progress_tracker(current_value, highest_value, initial_time, logger):
             logger = Settings.logger
 
         logger.info(
-            "Error occurred with Progress Tracker:\n{}".format(str(exc).encode("utf-8"))
+            "Error occurred with Progress Tracker:\n\t{}".format(
+                str(exc).encode("utf-8")
+            )
         )
 
 
 def close_dialog_box(browser):
-    """ Click on the close button spec. in the 'Likes' dialog box """
+    """Click on the close button spec. in the 'Likes' dialog box"""
 
     try:
         close = browser.find_element_by_xpath(
@@ -2255,7 +2352,7 @@ def close_dialog_box(browser):
 
 
 def parse_cli_args():
-    """ Parse arguments passed by command line interface """
+    """Parse arguments passed by command line interface"""
 
     AP_kwargs = dict(
         prog="InstaPy",
@@ -2450,11 +2547,11 @@ def get_query_hash(browser, logger, edge_followed_by):
     if hash:
         return hash[0]
     else:
-        logger.warn("Query Hash not found")
+        logger.warning("Query Hash not found")
 
 
 def file_handling(file):
-    """ Extracts text file's elements """
+    """Extracts text file's elements"""
     elements = []
     try:
         with open(file, "r") as f:
@@ -2492,3 +2589,39 @@ class CustomizedArgumentParser(ArgumentParser):
         will give the location of the 'argparse.py' file that have this method.
         """
         return []
+
+
+def get_additional_data(browser):
+    """
+    Get additional data object from page source
+    Idea and Code by alokkumarsbg
+
+    :param browser: The selenium webdriver instance
+    :return additional_data: Json data from window.__additionalData extracted from page source
+    """
+    additional_data = None
+    soup = BeautifulSoup(browser.page_source, "html.parser")
+    for text in soup(text=re.compile(r"window.__additionalDataLoaded")):
+        if re.search("^window.__additionalDataLoaded", text):
+            additional_data = json.loads(re.search("{.*}", text).group())
+            break
+
+    return additional_data
+
+
+def get_shared_data(browser):
+    """
+    Get shared data object from page source
+    Code by schealex
+
+    :param browser: The selenium webdriver instance
+    :return shared_data: Json data from window._sharedData extracted from page source
+    """
+    shared_data = None
+    soup = BeautifulSoup(browser.page_source, "html.parser")
+    for text in soup(text=re.compile(r"window._sharedData")):
+        if re.search("^window._sharedData", text):
+            shared_data = json.loads(re.search("{.*}", text).group())
+            break
+
+    return shared_data
