@@ -1,6 +1,18 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+###############################################################################
+# Note:
+# Update the xpaths with a generic form, try to avoid using xpaths like the
+# Browser is displaying; i.e.: /html/body/div[2]/div/div[2]/div[1]/div[2].
+# Usually the Browser use the most un-portable form and then it could be broken
+# after IG updates the DOM.
+#
+# Absolute: paths are the direct way to locate an element, "/html/body/div[2]"
+# Relative: starts with a double slash “//” that means it can start to search
+#           anywhere in the DOM structure, "//div[@class=’form-group’]"
+###############################################################################
+
 xpath = {}
 
 xpath["bypass_suspicious_login"] = {
@@ -18,7 +30,7 @@ xpath["dismiss_this_was_me"] = {
 
 xpath["class_selectors"] = {
     "likes_dialog_body_xpath": "//main",
-    "likes_dialog_close_xpath": "//div/button/span",
+    "likes_dialog_close_xpath": "//*[*[local-name()='svg']/@aria-label='Close']",
 }
 
 xpath["confirm_unfollow"] = {"button_xp": "//button[text()='Unfollow']"}
@@ -35,6 +47,11 @@ xpath["dismiss_notification_offer"] = {
     "dismiss_elem_loc": "//button[text()='Not Now']",
 }
 
+xpath["dismiss_save_information"] = {
+    "offer_elem_loc": "//*[contains(text(), 'Save Info')]",
+    "dismiss_elem_loc": "//*[contains(text(), 'Not Now')]",
+}
+
 xpath["extract_information"] = {
     "close_overlay": "//div/div[@role='dialog']",
     "one_pic_elem": "//section/main/article/div[1]/div/div[10]/div[3]/a/div",
@@ -69,10 +86,10 @@ xpath["get_comment_input"] = {
 }
 
 xpath["get_comments_on_post"] = {
-    "commenter_elem": "//h3/a",
-    "comments_block": "//div/div/h3/../../../..",
-    "like_button_full_XPath": "//div/span/button/span[@aria-label='Like']",
-    "unlike_button_full_XPath": "//div/span/button/span[@aria-label='Unlike']",
+    "commenter_elem": "//*[contains(@class,'FPmhX')]",
+    "comments_block": "//*[contains(@class,'EtaWk')]",
+    "like_button_full_XPath": "//*[*[local-name()='svg']/@aria-label='Like']",
+    "unlike_button_full_XPath": "//*[*[local-name()='svg']/@aria-label='Unlike']",
 }
 
 xpath["get_cord_location"] = {"json_text": "//body"}
@@ -82,7 +99,7 @@ xpath["get_following_status"] = {
                                   text()='Requested' or \
                                   text()='Follow' or \
                                   text()='Follow Back' or \
-                                  text()='Unblock']",
+                                  text()='Unblock' and not(ancestor::*/@role = 'presentation')]",
     "follow_span_XP_following": "//button/div/span[contains(@aria-label, 'Following')]",
 }
 
@@ -106,7 +123,7 @@ xpath["get_links_for_location"] = {
     "main_elem": "//main/article/div[2]",
 }
 
-xpath["get_links_from_feed"] = {"get_links": "//article/div[3]/div[2]/a"}
+xpath["get_links_from_feed"] = {"get_links": "//*[contains(@class,'c-Yi7')]"}
 
 xpath["get_links_for_tag"] = {
     "top_elements": "//main/article/div[1]",
@@ -136,14 +153,15 @@ xpath["get_users_through_dialog"] = {"find_dialog_box": "//body/div[4]/div/div[2
 xpath["is_private_profile"] = {"is_private": '//h2[@class="_kcrwx"]'}
 
 xpath["like_comment"] = {
-    "comments_block": "//div/div/h3/../../../..",
+    "comments_block": "//*[contains(@class,'EtaWk')]",
     "span_like_elements": "//span[@aria-label='Like']",
     "comment_like_button": "..",
 }
 
 xpath["like_image"] = {
-    "like": "//section/span/button/div/span[*[local-name()='svg']/@aria-label='Like']",
-    "unlike": "//section/span/button/div/span[*[local-name()='svg']/@aria-label='Unlike']",
+    "like": "//*[contains(@class, 'fr66n')]/button/div/*[*[local-name()='svg']/@aria-label='Like']/*",
+    "unlike": "//*[contains(@class, 'fr66n')]/button/div/*[*[local-name()='svg']/@aria-label='Unlike']/*",
+    "play": "//*/span[contains(@aria-label, 'Play')]",
 }
 
 xpath["like_from_image"] = {
@@ -164,6 +182,8 @@ xpath["login_user"] = {
     "add_phone_number": "//h2[text()='Add Your Phone Number']",
     "suspicious_login_attempt": "//p[text()='Suspicious Login Attempt']",
     "error_alert": "//p[@id='slfErrorAlert']",
+    "verification_code": "//input[@name='verificationCode']",
+    "confirm": "//button[text()='Confirm']",
 }
 
 xpath["open_comment_section"] = {
@@ -180,11 +200,13 @@ xpath["watch_story_for_tag"] = {"explore_stories": "//section/main/header/div[1]
 xpath["watch_story_for_user"] = {"explore_stories": "//section/main/div/header/div/div"}
 
 xpath["watch_story"] = {
-    "next_first": "/html/body/span/section/div/div/section/div[2]/button",
-    "next": "/html/body/span/section/div/div/section/div[2]/button[2]",
+    "next_first": "//*[contains(@class,'Cd8X1')]",
+    "next": "//*[contains(@class,'FhutL')]",
 }
 
 xpath["likers_from_photo"] = {
-    "liked_counter_button": "//div/article/div[2]/section[2]/div/div/a",
-    "second_counter_button": "//div/article/div[2]/section[2]/div/div/button",
+    "liked_counter_button": "//a[contains(@href,'liked_by')]/span",
+    "second_counter_button": "//a[contains(@href,'liked_by')]",
 }
+
+xpath["accept_igcookie_dialogue"] = {"accept_button": "//button[text()='Accept']"}
